/*
 * HTMLTokenMarker.java - HTML token marker
 * Copyright (C) 1998, 1999 Slava Pestov
 *
 * You may use and modify this package for any purpose. Redistribution is
 * permitted, in both source and binary form, provided that this notice
 * remains intact in all source distributions of this package.
 */

package ij.plugin.frame.jedit.tokenmarker;

import ij.plugin.frame.jedit.*;
import javax.swing.text.Segment;

/**
 * HTML token marker.
 *
 * @author Slava Pestov
 * @version $Id: HTMLTokenMarker.java,v 1.34 1999/12/13 03:40:29 sp Exp $
 */
public class XHTMLTokenMarker extends TokenMarker
{

    public static final byte JAVASCRIPT = Token.INTERNAL_FIRST;

    public XHTMLTokenMarker()
    {
            this(true,getKeywords());
    }

    public XHTMLTokenMarker(boolean js, KeywordMap keywords)
    {
            this.js = js;
            this.keywords = keywords;
    }

    public byte markTokensImpl(byte token, Segment line, int lineIndex)
    {
            char[] array = line.array;
            int offset = line.offset;
            lastOffset = offset;
            lastKeyword = offset;
            int length = line.count + offset;
            boolean backslash = false;

loop:		for(int i = offset; i < length; i++)
            {
                    int i1 = (i+1);

                    char c = array[i];

                    if(c == '\\')
                    {
                            backslash = !backslash;
                            continue;
                    }
                    switch(token)
                    {
                    case Token.NULL:
                            switch(c)
                            {
                            case '<':
                                    addToken(i - lastOffset,token);
                                    lastOffset = lastKeyword = i;
                                    if(SyntaxUtilities.regionMatches(false,
                                            line,i1,"!--"))
                                    {
                                            i += 3;
                                            token = Token.COMMENT1;
                                    }
                                    else
                                    {
                                        if(backslash)
                                           backslash = false;
                                        doKeyword(line,i,c);
                                    }
                                    break;
                            case '"':
                                    doKeyword(line,i,c);
                                    if(backslash)
                                            backslash = false;
                                    else
                                    {
                                            addToken(i - lastOffset,token);
                                            token = Token.LITERAL1;
                                            lastOffset = lastKeyword = i;
                                    }
                                    break;
                            case '&':
                                        addToken(i - lastOffset,token);
                                        lastOffset = lastKeyword = i;
                                        token = Token.KEYWORD2;
                                        break;
                            default:
                                    backslash = false;
                                    if(!Character.isLetterOrDigit(c)
                                            && c != '_')
                                            doKeyword(line,i,c);
                                    break;
                            }
                            break;
                        case Token.KEYWORD1: // Inside a tag
                                backslash = false;
                                if(c == '>')
                                {
                                        addToken(i1 - lastOffset,token);
                                        lastOffset = lastKeyword = i1;
                                        token = Token.NULL;
                                }
                                break;
                            case Token.KEYWORD2: // Inside an entity
                                    backslash = false;
                                    if(c == ';')
                                    {
                                            addToken(i1 - lastOffset,token);
                                            lastOffset = lastKeyword = i1;
                                            token = Token.NULL;
                                            break;
                                    }
                                    break;
                        case Token.COMMENT1: // Inside a comment
                                backslash = false;
                                if(SyntaxUtilities.regionMatches(false,line,i,"-->"))
                                {
                                        addToken((i + 3) - lastOffset,token);
                                        lastOffset = lastKeyword = i + 3;
                                        token = Token.NULL;
                                }
                                break;
                    case Token.LITERAL1:
                            if(backslash)
                                    backslash = false;
                            else if(c == '"')
                            {
                                    addToken(i1 - lastOffset,token);
                                    token = Token.NULL;
                                    lastOffset = lastKeyword = i1;
                            }
                            break;
                    case Token.LITERAL2:
                            if(backslash)// JavaScript '...'
                                    backslash = false;
                                break;
                    default:
                            throw new InternalError("Invalid state: "
                                    + token);
                    }
            }

            if(token == Token.NULL)
                    doKeyword(line,length,'\0');

            switch(token)
            {
            case Token.LITERAL1:
            case Token.LITERAL2:
                    addToken(length - lastOffset,Token.INVALID);
                    token = Token.NULL;
                    break;
            case Token.KEYWORD2:
                    addToken(length - lastOffset,token);
                    if(!backslash)
                            token = Token.NULL;
                    /*case JAVASCRIPT:
                            doKeyword(line,length,'\0');
                            addToken(length - lastOffset,Token.NULL);
                            break;*/
            default:
                    addToken(length - lastOffset,token);
                    break;
            }

            return token;
    }

    public static KeywordMap getKeywords()
    {
            if(htmlsyntaxKeywords == null)
            {
                htmlsyntaxKeywords = new KeywordMap(true);
                    htmlsyntaxKeywords.add("a",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("abbr",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("above",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("accept",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("accept-charset",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("accesskey",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("acronym",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("action",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("address",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("align",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("alink",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("alt",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("applet",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("area",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("axis",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("b",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("background",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("base",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("basefont",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("bdo",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("behavior",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("below",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("bgcolor",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("bgproperties",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("bgsound",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("big",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("blockquote",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("body",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("border",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("bordercolor",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("br",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("button",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("caption",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("cellpadding",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("center",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("charoff",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("charset",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("cite",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("class",Token.KEYWORD2);
                    htmlsyntaxKeywords.add("classid",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("clear",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("clip",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("code",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("codebase",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("col",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("colgroup",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("color",Token.KEYWORD2);
                    htmlsyntaxKeywords.add("cols",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("colspan",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("compact",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("coords",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("data",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("datetime",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("dd",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("declare",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("defer",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("del",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("dfn",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("dir",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("direction",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("disabled",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("div",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("dl",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("doctype",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("dt",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("em",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("embed",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("enctype",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("equiv",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("face",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("fieldset",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("font",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("for",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("form",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("frame",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("frameborder",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("frameset",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("framespacing",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("gutter",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("h1",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("h2",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("h3",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("h4",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("h5",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("h6",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("head",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("headers",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("height",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("hr",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("href",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("hreflang",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("hspace",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("html",Token.KEYWORD1);
                    //htmlsyntaxKeywords.add("http-equiv",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("http",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("http public",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("i",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("id",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("iframe",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("ilayer",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("img",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("input",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("ins",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("isindex",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("ismap",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("kbd",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("label",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("lang",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("language",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("layer",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("left",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("leftmargin",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("legend",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("li",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("link",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("listing",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("longdesc",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("map",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("marginheight",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("marginwidth",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("marquee",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("maxlength",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("media",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("menu",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("meta",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("method",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("multicol",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("multiple",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("name",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("nobr",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("noembed",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("noframes",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("noresize",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("noscript",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("noshade",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("nowrap",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("object",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("ol",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("onclick",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("ondblclick",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onkeydown",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onkeypress",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onkeyup",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onmousedown",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onmousemove",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onmouseout",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onmouseover",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("onmouseup",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("optgroup",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("option",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("p",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("param",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("plaintext",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("pre",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("profile",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("prompt",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("public",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("q",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("readonly",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("rel",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("rev",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("rows",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("rowspan",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("rules",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("s",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("samp",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("scheme",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("script",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("scrollamount",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("scrolling",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("select",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("selected",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("shape",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("size",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("small",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("spacer",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("span",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("src",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("standby",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("strike",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("strong",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("style",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("sub",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("summary",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("sup",Token.KEYWORD1);

                    htmlsyntaxKeywords.add("tabindex",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("table",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("target",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("tbody",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("td",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("text",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("textarea",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("tfoot",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("th",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("thead",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("title",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("top",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("topmargin",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("tr",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("tt",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("type",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("u",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("ul",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("usemap",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("valign",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("value",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("var",Token.KEYWORD1);
                    htmlsyntaxKeywords.add("visibility",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("vlink",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("vspace",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("wbr",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("width",Token.KEYWORD3);
                    htmlsyntaxKeywords.add("wrap",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("xmp",Token.KEYWORD3);

                    htmlsyntaxKeywords.add("z-index",Token.KEYWORD3);

            }
            return htmlsyntaxKeywords;
    }

    // private members
    private static KeywordMap htmlsyntaxKeywords;

    private boolean js;
    private KeywordMap keywords;
    private int lastOffset;
    private int lastKeyword;

    private boolean doKeyword(Segment line, int i, char c)
    {
            int i1 = i+1;

            int len = i - lastKeyword;
            byte id = keywords.lookup(line,lastKeyword,len);
            if(id != Token.NULL)
            {
                    if(lastKeyword != lastOffset)
                            addToken(lastKeyword - lastOffset,Token.NULL);
                    addToken(len,id);
                    lastOffset = i;
            }
            lastKeyword = i1;
            return false;
    }
}
