/*
 * $Id: ControlBarPanel.java $
 *
 * Copyright (c) 2001-2011 by the authors of ImageJ2x
 * All rights reserved.
 *
 *
 */

package ij2x.plugin;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;

import java.net.URL;

import java.text.DateFormat;

import java.util.Date;
import java.util.MissingResourceException;
import java.util.Locale;
import java.util.ResourceBundle;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.BevelBorder;
import javax.swing.BorderFactory;
import javax.swing.border.Border;

import ij.IJ;
import ij.ImageJ;
import ij.ImagePlus;
import ij.Prefs;
import ij.WindowManager;
import ij.plugin.PlugIn;
import javax.swing.AbstractAction;
import javax.swing.Action;

public class ControlBarPanel extends JPanel implements PlugIn, ActionListener, MouseListener, MouseMotionListener, Runnable{

    protected ImagePlus imp;
    protected ImageJ ij;
    private int offset=60, offsetHeight;
    public JButton infoButton, draggedButton, closeButton, undecoratedButton, action;
    protected JButton newButton, openButton, undoButton, copyButton, cutButton, pasteButton
            , clearButton, fillButton, drawButton, roicolorButton, colorpickerButton
            , selectButton, nonselectButton
            , splineButton, duplicateButton, captureButton, rotateButton, clockButton, specifyButton;
    private BevelBorder bb ;//new BevelBorder for LOWER or RAISED type
    private Border lightGrayBorder = BorderFactory.createBevelBorder(bb.RAISED, Color.lightGray, Color.lightGray);
    private long mouseDownTime;
    private static int current;
    public static final int DOUBLE_CLICK_THRESHOLD = 150;
    public JPanel controlBarPanel=new JPanel();
    public JLabel spaceLabel, clockLabel, label;
    public JPanel panel=new JPanel();
    protected Date date = new Date();

    private boolean running=false;
    private Thread timer;
    private String time;
    private DateFormat timeFormat;
    private static boolean isparent = false;
    private static ResourceBundle resources;
    private static String fileType = ".gif";
    private static String dir;
    private int index = 0;
    private static int ARRAYLENGTH = 20;
    private static int PARTSLENGTH = 3;

    static {
        try {
            resources = ResourceBundle.getBundle("ij2x.plugin.resources.ControlBarPanel",
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("resources/ControlBarPanel.properties not found");
            System.exit(1);
        }
    }


    public ControlBarPanel() {
        ij = IJ.getInstance();
        //dir = Prefs.getString("icondir");
        //controlBarPanel=createControlBar();
        timeFormat= DateFormat.getTimeInstance(DateFormat.DEFAULT);
        start();
}

    public void run(String arg){
        ij = IJ.getInstance();
        controlBarPanel=createControlBar();
        //timeFormat= DateFormat.getTimeInstance(DateFormat.DEFAULT);
        //start();
   }

    /*protected String getResourceString(String nm) {
        String str;
        try {
            str = resources.getString(nm);
        } catch (MissingResourceException mre) {
            str = null;
        }
        return str;
    }*/

    /*protected URL getResource(String key) {
         String name = getResourceString(key);
         if (name != null) {
             URL url = this.getClass().getResource(name);
             return url;
         }
         return null;
     }*/

     /**
      * Take the given string and chop it up into a series
      * of strings on whitespace boundaries.  This is useful
      * for trying to get an array of strings out of the
      * resource file.
      */
     /*protected String[] tokenize(String input) {
         Vector<String> v = new Vector<String>();
         StringTokenizer t = new StringTokenizer(input);
         String cmd[];

         while (t.hasMoreTokens())
             v.addElement(t.nextToken());
         cmd = new String[v.size()];
         for (int i = 0; i < cmd.length; i++)
             cmd[i] = v.elementAt(i);

         return cmd;
     }*/
     /**
      * Create the toolbar.  By default this reads the
      * resource file for the definition of the toolbar.
      */
     /*private Component createToolbar() {
         toolbar = new JToolBar();
         boolean isbutton=true;
         String[] toolKeys = tokenize(getResourceString("toolbar"));
         for (int i = 0; i < toolKeys.length; i++) {
             if (toolKeys[i].equals("-")) {
                 toolbar.add(Box.createHorizontalStrut(5));
             } else {
                 String bot= getResourceString(toolKeys[i] + "Button");
                 if(bot!=null){
                     Boolean shifts = Boolean.valueOf(bot);
                     isbutton = shifts.booleanValue();
                 }//else isbutton=false;
                 toolbar.add(createTool(toolKeys[i], isbutton));
             }
         }
         toolbar.add(Box.createHorizontalGlue());
         return toolbar;
     }*/


    public JPanel createControlBar(){

        java.awt.FlowLayout flayout = new java.awt.FlowLayout(java.awt.FlowLayout.LEFT);
        flayout.setHgap(0);
        flayout.setVgap(0);
        panel.setLayout(flayout);
        panel.setBorder(lightGrayBorder);
        clockLabel=createLabel(clockLabel, "", "00:00:00");
        this.setDate(new Date());
        clockLabel.setBorder(lightGrayBorder);
        dir = Prefs.getString("icondir");

        //toolbar= image open undo copy cut paste clear fill draw duplicate selectall deselect fitspline threecolor jcolopicker cupture info  rotate scale spezifyroi
        //          0      1    2    3   4    5     6     7    8      9        10        11       12        13          14          15    16    17     18     19

        // This code ensures that all the pixels in the image are loaded
        infoButton=createButton(infoButton, "Show Info ...");
        infoButton.setIcon(createIcon("s_info"));
        infoButton.setRolloverIcon(createIcon("s_info_ro"));
        infoButton.setPressedIcon(createIcon("s_info_press"));

        newButton=createButton(newButton, "New Image");
        newButton.setIcon(createIcon("s_new"));
        newButton.setRolloverIcon(createIcon("s_new_ro"));
        newButton.setPressedIcon(createIcon("s_new_press"));

        openButton=createButton(openButton, "Open Image");
        openButton.setIcon(createIcon("s_open"));
        openButton.setRolloverIcon(createIcon("s_open_ro"));
        openButton.setPressedIcon(createIcon("s_open_press"));

        duplicateButton=createButton(duplicateButton, "Duplicate Image");
        duplicateButton.setIcon(createIcon("s_duplicate"));
        duplicateButton.setRolloverIcon(createIcon("s_duplicate_ro"));
        duplicateButton.setPressedIcon(createIcon("s_duplicate_press"));

        undoButton=createButton(undoButton, "Undo Action");
        undoButton.setIcon(createIcon("s_undo"));
        undoButton.setRolloverIcon(createIcon("s_undo_ro"));
        undoButton.setPressedIcon(createIcon("s_undo_press"));

        copyButton=createButton(copyButton, "Copy Shape");
        copyButton.setIcon(createIcon("s_copy"));
        copyButton.setRolloverIcon(createIcon("s_copy_ro"));
        copyButton.setPressedIcon(createIcon("s_copy_press"));

        cutButton=createButton(cutButton, "Cut Shape");
        cutButton.setIcon(createIcon("s_cut"));
        cutButton.setRolloverIcon(createIcon("s_cut_ro"));
        cutButton.setPressedIcon(createIcon("s_cut_press"));

        pasteButton=createButton(pasteButton, "Paste Shape");
        pasteButton.setIcon(createIcon("s_paste"));
        pasteButton.setRolloverIcon(createIcon("s_paste_ro"));
        pasteButton.setPressedIcon(createIcon("s_paste_press"));

        clearButton=createButton(clearButton, "Clear Shape");
        clearButton.setIcon(createIcon("s_clear"));
        clearButton.setRolloverIcon(createIcon("s_clear_ro"));
        clearButton.setPressedIcon(createIcon("s_clear_press"));

        fillButton=createButton(fillButton, "Fill Shape");
        fillButton.setIcon(createIcon("s_fill"));
        fillButton.setRolloverIcon(createIcon("s_fill_ro"));
        fillButton.setPressedIcon(createIcon("s_fill_press"));

        drawButton=createButton(drawButton, "Draw Shape");
        drawButton.setIcon(createIcon("s_pencil"));
        drawButton.setRolloverIcon(createIcon("s_pencil_ro"));
        drawButton.setPressedIcon(createIcon("s_pencil_press"));

        selectButton=createButton(selectButton, "Select Image");
        selectButton.setIcon(createIcon("s_select"));
        selectButton.setRolloverIcon(createIcon("s_select_ro"));
        selectButton.setPressedIcon(createIcon("s_select_press"));

        nonselectButton=createButton(nonselectButton, "Non Select Image");
        nonselectButton.setIcon(createIcon("s_nonselect"));
        nonselectButton.setRolloverIcon(createIcon("s_nonselect_ro"));
        nonselectButton.setPressedIcon(createIcon("s_nonselect_press"));

        splineButton=createButton(splineButton, "Fit Spline");
        splineButton.setIcon(createIcon("s_spline"));
        splineButton.setRolloverIcon(createIcon("s_spline_ro"));
        splineButton.setPressedIcon(createIcon("s_spline_press"));

        roicolorButton=createButton(roicolorButton, "Shape Color");
        roicolorButton.setIcon(createIcon("s_shapecolor"));
        roicolorButton.setRolloverIcon(createIcon("s_shapecolor_ro"));
        roicolorButton.setPressedIcon(createIcon("s_shapecolor_press"));

        colorpickerButton=createButton(colorpickerButton, "JColor Picker");
        colorpickerButton.setIcon(createIcon("s_picker"));
        colorpickerButton.setRolloverIcon(createIcon("s_picker_ro"));
        colorpickerButton.setPressedIcon(createIcon("s_picker_press"));

        captureButton=createButton(captureButton, "Capture Screen");
        captureButton.setIcon(createIcon("s_captscreen"));
        captureButton.setRolloverIcon(createIcon("s_captscreen_ro"));
        captureButton.setPressedIcon(createIcon("s_captscreen_press"));

        specifyButton=createButton(specifyButton, "Spezify Roi");
        specifyButton.setIcon(createIcon("s_spezifyroi"));
        specifyButton.setRolloverIcon(createIcon("s_spezifyroi_ro"));
        specifyButton.setPressedIcon(createIcon("s_spezifyroi_press"));

        clockButton = createButton(clockButton, "Clock Button");
        clockButton.setIcon(createIcon("s_clock"));
        clockButton.setPressedIcon(createIcon("s_clock_press"));
        clockButton.setRolloverIcon(createIcon("s_clock_ro"));

        spaceLabel = new JLabel();

        offsetHeight=(int)infoButton.getPreferredSize().getHeight();

        spaceLabel.setMinimumSize(new Dimension(offset, offsetHeight+1));
        spaceLabel.setMaximumSize(new Dimension(offset, offsetHeight+1));
        spaceLabel.setPreferredSize(new Dimension(offset, offsetHeight+1));
        spaceLabel = new JLabel("", JLabel.CENTER);
        offsetHeight=(int)infoButton.getPreferredSize().getHeight();
        spaceLabel.setMinimumSize(new Dimension(offset, offsetHeight));
        spaceLabel.setMaximumSize(new Dimension(offset, offsetHeight));
        spaceLabel.setPreferredSize(new Dimension(offset, offsetHeight));
        spaceLabel.setBorder(lightGrayBorder);
        panel.add(newButton);
        panel.add(openButton);
        panel.add(undoButton);
        panel.add(copyButton);
        panel.add(cutButton);
        panel.add(pasteButton);
        panel.add(clearButton);
        panel.add(fillButton);
        panel.add(drawButton);
        panel.add(roicolorButton);
        panel.add(colorpickerButton);
        panel.add(specifyButton);
        panel.add(selectButton);
        panel.add(nonselectButton);
        panel.add(splineButton);
        panel.add(duplicateButton);
        panel.add(captureButton);
        //panel.add(rotateButton);
        panel.add(infoButton);
        panel.add(clockButton);

        JPanel stbPanel = new JPanel();
        stbPanel.setLayout(new BorderLayout());
        stbPanel.add(BorderLayout.WEST, panel);
        //stbPanel.add(BorderLayout.CENTER, spaceLabel);
        //stbPanel.add(BorderLayout.EAST, clockLabel);
        return stbPanel;
     }
     private JLabel createLabel(JLabel label, String tooltip, String text){
         label = new JLabel(text);
         label.setToolTipText(tooltip);
         label.addMouseListener(new MouseListener(){
         public void mousePressed(MouseEvent e){
             if (!e.isMetaDown()){
                 labelEvent(e);
             }
         }
         public void mouseReleased(MouseEvent e) {}
         public void mouseExited(MouseEvent e) {}
         public void mouseClicked(MouseEvent e) {}
         public void mouseEntered(MouseEvent e) {}
         });
     return label;
     }

     public void labelEvent(MouseEvent e){
         JLabel label = (JLabel)e.getSource();
         JFrame[] editFrame = WindowManager.getNonImageWindows();
         if(editFrame.length > 1){
             for (int i = 0; i < editFrame.length; i++) {
                 //IJ.log("" + editFrame[i].getClass().getName());
                 if (editFrame[i].getClass().getName().equals(
                         "com.ibm.JCalendar")) {
                     isparent = true;
                     editFrame[i].setVisible(false);
                     WindowManager.removeWindow(editFrame[i]);
                 } else
                     isparent = false;
             }
         }else
         if (editFrame.length == 1 && editFrame[0].getClass().getName().equals(
                 "com.ibm.JCalendar")) {
             isparent = true;
             editFrame[0].setVisible(false);
             WindowManager.removeWindow(editFrame[0]);
         }else isparent = false;
        if(label==clockLabel && !isparent){
           IJ.runPlugIn("com.ibm.JCalendar", "");
       }
     }
     private JButton createButton(JButton action, String tooltip, Image[] image){
           action = new JButton();
           action.setToolTipText(tooltip);
           action.addActionListener(this);
           action.addMouseListener(this);
           action.addMouseMotionListener(this);
           setBorderlessButton(action);
           setIcon(action, image);
     return action;
     }

     private JButton createButton(JButton action, String tooltip){
           action = new JButton();
           action.setToolTipText(tooltip);
           action.addActionListener(this);
           action.addMouseListener(this);
           action.addMouseMotionListener(this);
           setBorderlessButton(action);
           //setIcon(action, image);
     return action;
     }


     private Icon createIcon(String key){
         Icon icon = null;
         String fileName=key + getFileType();
         if(getImageDir() == null){
          dir="images/";
         }
         String path = getImageDir()+fileName;
        try {
            URL url = this.getClass().getResource(path);
            Image image = Toolkit.getDefaultToolkit().createImage(url);
            if (url != null) {
                icon = new ImageIcon(image);
              }
            } catch (Exception ex) {
          }
            return icon;
     }

     private String getFileType(){
            return fileType;
     }
     private String getImageDir(){
            return dir;
     }
     private void setIcon(JButton action, Image[] image){
           action.setIcon(new ImageIcon(image[0]));
           action.setRolloverIcon(new ImageIcon(image[1]));
           action.setPressedIcon(new ImageIcon(image[2]));
     }
     void setButtonDim(JButton action, Dimension maxDim, Dimension preDim, Dimension minDim){
           action.setMaximumSize(maxDim);
           action.setPreferredSize(preDim);
           action.setMinimumSize(minDim);
   }

     void setBorderlessButton(JButton button){
           button.setMargin(new Insets(0, 0, 0, 0));
           button.setBorderPainted(false);
           button.setContentAreaFilled(false);
           button.setFocusPainted(false);
       }

       public void mousePressed(MouseEvent e) {
           action = (JButton)e.getSource();
           int x = e.getX();
           boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
           mouseDownTime = System.currentTimeMillis();
                if (!doubleClick) {
                }
        }

        public void mouseReleased(MouseEvent e) {}
        public void mouseExited(MouseEvent e) {}
        public void mouseClicked(MouseEvent e) {}
        public void mouseEntered(MouseEvent e) {}

        public void mouseDragged(MouseEvent e) {}
        public void mouseMoved(MouseEvent e) {
            JButton action = (JButton)e.getSource();
            int x = e.getX();
            boolean doubleClick = (System.currentTimeMillis()-mouseDownTime)<=DOUBLE_CLICK_THRESHOLD;
            mouseDownTime = System.currentTimeMillis();
                 if (!false) {
                     if(action == infoButton){
                         IJ.showStatus("Command: Show Info...");
                     }
                     else if(action == openButton){
                         IJ.showStatus("Command: Open...");
                     }
                     else if(action == newButton){
                         IJ.showStatus("Command: Image...");
                     }
                     else if(action == undoButton){
                         IJ.showStatus("Command: Undo");
                     }
                     else if(action == copyButton){
                         IJ.showStatus("Command: Copy");
                     }
                     else if(action == cutButton){
                         IJ.showStatus("Command: Cut");
                     }
                     else if(action == pasteButton){
                         IJ.showStatus("Command: Paste");
                     }
                     else if(action == clearButton){
                         IJ.showStatus("Command: Clear");
                     }
                     else if(action == fillButton){
                         IJ.showStatus("Command: Fill");
                     }
                     else if(action == drawButton){
                         IJ.showStatus("Command: Draw");
                     }
                     else if(action == duplicateButton){
                         IJ.showStatus("Command: Duplicate...");
                     }
                     /*else if(action == scaleButton){
                         IJ.showStatus("Command: Scale...");
                     }*/
                     else if(action == selectButton){
                         IJ.showStatus("Command: Select All");
                     }
                     else if(action == nonselectButton){
                         IJ.showStatus("Select None");
                     }
                     /*else if(action == rotateButton){
                         IJ.showStatus("Command: Arbitrarily...");
                     }*/
                     else if(action == splineButton){
                         IJ.showStatus("Command: Fit Spline");
                     }
                     else if(action == roicolorButton){
                             IJ.showStatus("Command: Roi Color");
                     }
                     else if(action == colorpickerButton){
                         IJ.showStatus("Command: JColor Picker...");
                     }
                     else if(action == captureButton){
                         IJ.showStatus("Command: Capture Screen ");
                     }
                     else if (action == draggedButton) {
                         IJ.showStatus("Move...");
                     }
                     else if (action == closeButton) {
                         IJ.showStatus("Decorated Frame");
                     }
                     else if (action == undecoratedButton) {
                         IJ.showStatus("Undecorated Frame");
                     }

                 }
        }
        // Action listener method
        public void actionPerformed(ActionEvent e){
            try {
                if(action == infoButton){
                    IJ.doCommand("Show Info...");
                }
                else if(action == openButton){
                    IJ.doCommand("Open...");
                }
                else if(action == newButton){
                    IJ.doCommand("Image...");
                    //IJ.run("Image...", "");
                }
                else if(action == undoButton){
                    IJ.doCommand("Undo");
                }
                else if(action == copyButton){
                    IJ.doCommand("Copy");
                }
                else if(action == cutButton){
                    IJ.doCommand("Cut");
                }
                else if(action == pasteButton){
                    IJ.doCommand("Paste");
                }
                else if(action == clearButton){
                    IJ.doCommand("Clear");
                }
                else if(action == fillButton){
                    IJ.doCommand("Fill");
                }
                else if(action == drawButton){
                    IJ.doCommand("Draw");
                }
                else if(action == duplicateButton){
                    IJ.doCommand("Duplicate...");
                }
                else if(action == selectButton){
                    IJ.doCommand("Select All");
                }
                else if(action == nonselectButton){
                    IJ.doCommand("Select None");
                }
                else if(action == splineButton){
                    IJ.doCommand("Fit Spline");
                }
                else if(action == colorpickerButton){
                    if(!IJ.isWindows()){
                        IJ.runPlugIn("ij2x.plugin.frame.JColorPicker", "");
                    }else{
                        IJ.runPlugIn("ij2x.plugin.frame.JColorPicker", "");
                    }
                }
                else if(action == roicolorButton){
                        IJ.runPlugIn("ij2x.plugin.ThreeColor","");
                }
                else if(action == captureButton){
                    IJ.doCommand("Capture Screen");
                }
                else if(action == specifyButton){
                    IJ.doCommand("Specify...");
                }
                else if(action == clockButton){
                    JFrame[] editFrame = WindowManager.getNonImageWindows();
                    if(editFrame.length > 1){
                        for (int i = 0; i < editFrame.length; i++) {
                            //IJ.log("" + editFrame[i].getClass().getName());
                            if (editFrame[i].getClass().getName().equals(
                                    "com.ibm.JCalendar")) {
                                isparent = true;
                                editFrame[i].setVisible(false);
                                WindowManager.removeWindow(editFrame[i]);
                            } else
                                isparent = false;
                        }
                    }else
                    if (editFrame.length == 1 && editFrame[0].getClass().getName().equals(
                            "com.ibm.JCalendar")) {
                        isparent = true;
                        editFrame[0].setVisible(false);
                        WindowManager.removeWindow(editFrame[0]);
                    }else isparent = false;
                   if(!isparent){
                      IJ.runPlugIn("com.ibm.JCalendar", "");
                  }
                }
            } catch (Exception ex) {

            }
        }

        public static final String cutAction = "cut";
        public static final String copyAction = "copy";
        public static final String openAction = "open";
        private Action[] defaultActions = {
                                          new CopyAction(),
                                          new CutAction(),
                                          new OpenAction()
        };

        class OpenAction extends AbstractAction {

            OpenAction() {
                super(openAction);
            }

            public void actionPerformed(ActionEvent e) {
                IJ.doCommand("Open...");
            }
        }

        class CopyAction extends AbstractAction {

            CopyAction() {
                super(copyAction);
            }

            public void actionPerformed(ActionEvent e) {
                IJ.doCommand("Copy");
            }
        }

        class CutAction extends AbstractAction {

            CutAction() {
                super(cutAction);
            }

            public void actionPerformed(ActionEvent e) {
                IJ.doCommand("Cut");
            }
        }


        public void setDate(Date date) {
            this.date = date;
        }

        /* geter methods for the timeFormat value*/
        public String getTimeFormatToString() {
            return time;
        }

        /* seter methods for the timeFormat value*/
        public void setTimeFormatToString(String time) {
            this.time = time;
        }

        /* interface Runnable methods */
        public void start() {
            running = true;
            if (timer == null)
                timer = new Thread(this);
            timer.start();
        }

        public void stop() {
            timer = null;
            notify();
        }

        public void run() {
            while (running) {
                String tFormat = timeFormat.format(new Date());
                setTimeFormatToString(tFormat);
                this.setDate(new Date());
                try {
                    Thread.sleep(200);
                    if(clockLabel != null){
                        clockLabel.setText(tFormat);
                        clockLabel.setToolTipText("Die Zeit:" + tFormat);
                        clockButton.setToolTipText("Die Zeit:" + tFormat);
                    }
                 } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            timer = null;
        }

}
